package org.barney.greenfoot.core;
import greenfoot.*;  // (World, Actor, GreenfootImage, Greenfoot and MouseInfo)

/**
 * Die Klasse <code>Collector</code> definiert Methoden zum Sammeln von Objekten.
 * Alle Objekte, die ein Untertyp von <code>Collectable</code> sind, können durch diese Methoden aufgesammelt werden.
 * 
 * @author Jan-Niklas Brandes 
 * @version 1.0
 */
public class Collector extends Operator
{
    /**
     * Gibt zurck, ob auf dem Feld dieses Objektes etwas Sammelbares liegt.
     * @return  Ob ein aufsammelbares Objekt auf dem Feld liegt.
     */
    public boolean collectableOnField(){
        return specificOnField(Collectable.class);
    }
    /**
     * Gibt zurck, ob auf dem Feld diess Objektes mindestens ein Objekt der angegebenen Klasse liegt.
     * @param  cls Die Klasse der Objekte, nach denen geprft werden soll.
     * @return  Ob ein Objekt der Klasse auf dem Feld liegt.
     */
    public <A extends Collectable> boolean specificOnField(Class<A> cls){
        return this.getIntersectingObjects(cls).size()>0;
    }
    /**
     * Sammelt ein Objekt der angegebenen Klasse auf.
     * @param  cls Die Klasse des zu sammelnden <code>Collectable</code>.
     * @return  Die Zahl der aufgesammelten Objekte.
     */
    public <A extends Collectable> int collectSpecific(Class<A> cls){
        A c = null;
        if((c = (A)this.getOneIntersectingObject(cls)) != null){
            this.getWorld().removeObject(c);
            return 1;
        }else{
            return 0;
        }
    }
    /**
     * Sammelt ein <code>Collectable</code> auf.
     * @return  Die Zahl der aufgesammelten Objekte.
     */
    public int collect(){
        return collectSpecific(Collectable.class);
    }
    /**
     * Sammelt alle <code>Collectable</code> auf.
     * @return  Die Zahl der aufgesammelten Objekte.
     */
    public int collectAll(){
        return collectAllSpecific(Collectable.class);
    }
    /**
     * Sammelt alle Objekte der angegebenen Klasse auf.
     * @param  cls Die Klasse der aufzusammelnden Objekte.
     * @return  Die Zahl der aufgesammelten Objekte.
     */
    public <A extends Collectable> int collectAllSpecific(Class<A> cls){
        int collected = this.getIntersectingObjects(cls).size();
        this.getWorld().removeObjects(this.getIntersectingObjects(cls));
        return collected;
    }
}
