package org.barney.greenfoot.core;
import org.barney.greenfoot.world.Coordinate;
import greenfoot.*;  // (World, Actor, GreenfootImage, Greenfoot and MouseInfo)

/**
 * Die Klasse <code>Operator</code> definiert Methoden fr Objekte, die sich in einer Welt bewegen und mit Objekten interagieren.
 * Unterklassen erhalten so weitere Bewegungs-, Erkennungs- und Interaktionsmethoden.
 * 
 * @author Jan-Niklas Brandes 
 * @version 1.0
 */
public class Operator extends Actor
{
    /**
     * Reprsentiert den Gradwert von Osten.
     */
    private static final int EAST = 0;
    /**
     * Reprsentiert den Gradwert von Westen.
     */
    private static final int WEST = 180;
    /**
     * Reprsentiert den Gradwert von Norden.
     */
    private static final int NORTH = 270;
    /**
     * Reprsentiert den Gradwert von Sden.
     */
    private static final int SOUTH = 90;
    
    /**
     * Gibt ein Objekt zurck, dass auf demselben Feld liegt und eine Instanz der Klasse <code>cls</code> ist.
     * @param  cls Die Klasse des zu untersuchenden Objekts.
     * @return  Ein Objekt der angegebenen Klasse, dass sich auf demselben Feld wie der <code>Operator</code> befindet.
     */
    public Actor getOneIntersectingObject(Class cls){
        return super.getOneIntersectingObject(cls);
    }
    /**
     * Bewegt diesen <code>Operator</code> ein Feld vorwrts.
     */
    public void move()
    {
        if (!canMove()) {
            return;
        }
        switch(getRotation()) {
            case SOUTH :
                setLocation(getX(), getY() + 1);
                break;
            case EAST :
                setLocation(getX() + 1, getY());
                break;
            case NORTH :
                setLocation(getX(), getY() - 1);
                break;
            case WEST :
                setLocation(getX() - 1, getY());
                break;
        }
    }

    /**
     * Gibt zurck, ob sich dieser <code>Operator</code> vorwrts bewegen kann.
     * @return  Ob sich dieser <code>Operator</code> vorwrts bewegen kann.
     */
    public boolean canMove()
    {
        World myWorld = getWorld();
        int x = getX();
        int y = getY();
        switch(getRotation()) {
            case SOUTH :
                y++;
                break;
            case EAST :
                x++;
                break;
            case NORTH :
                y--;
                break;
            case WEST :
                x--;
                break;
        }
        // test for outside border
        if (x >= myWorld.getWidth() || y >= myWorld.getHeight()) {
            return false;
        }
        else if (x < 0 || y < 0) {
            return false;
        }else if(getWorld().getObjectsAt(x, y, Obstacle.class).size()>0){
            return false;
        }
        return true;
    }
    
    /**
     * Dreht diesen <code>Operator</code> um 90 nach links.
     */
    public void turnLeft(){
        this.setRotation((this.getRotation()-90)%360);
    }
    /**
     * Dreht diesen <code>Operator</code> um 90 nach rechts.
     */
    public void turnRight(){
        this.setRotation((this.getRotation()+90)%360);
    }
    /**
     * Bewegt diesen <code>Operator</code> zu den angegebenen Koordinaten
     * @param  point Die Koordnate des Feldes, auf das sich dieser <code>Operator</code> soll.
     */
    public void moveTo(Coordinate point){
        if(point != null){
            if(point.getX() < this.getWorld().getWidth() && point.getX() >= 0 && point.getY() < this.getWorld().getHeight() && point.getY() >= 0 && this.getWorld().getObjectsAt(point.getX(), point.getY(), Obstacle.class).size() == 0){
                this.turnTowards(point.getX(), point.getY());
                this.setLocation(point.getX(), point.getY());
            }
        }
    }
}
