package org.barney.greenfoot.extensions;
import org.barney.greenfoot.pathfinding.*;
import org.barney.greenfoot.world.*;
import org.barney.greenfoot.core.*;
import greenfoot.*;
import java.util.*;
/**
 * Stellt Methoden fr die Berechnung eines optimalen Pfades bereit.
 */
public class Pathfinder extends Extension implements PathfindingAlgorithm
{
    /**
     * Hlt Referenzen auf Punkte, die noch expandiert werden mssen.
     */
    public PriorityQueue openList;
    /**
     * Hlt Referenzen auf Punkte, die bereits expandiert sind.
     */
    public HashSet closedList;
    private Pathfinder instance;
    private Stack<Coordinate> way;
    private PathfindingAlgorithm mode;
    private static PathfindingAlgorithm ASTAR;
    private static PathfindingAlgorithm DSTAR;
    public boolean gotPath;
    /**
     * Instantiiert ein <code>Pathfinder</code>-Modul.
     * @param  operator Der <code>Operator</code>, der dieses Modul nutzen will.
     */
    public Pathfinder(Operator operator){
        super(operator);
        instance = this;
        ASTAR = new AStarAlgorithm();
        DSTAR = null;
        mode = ASTAR;
        gotPath = false;
    }
    public void init(){
        mode = ASTAR;
        mode.init();
    }
    public boolean computePathTo(int x, int y){
        return mode.computePathTo(x, y);
    }
    public Coordinate step(){
        return mode.step();
    }
    public void setFocussed(boolean focussed){
        mode.setFocussed(focussed);
    }
    public boolean getFocussed(){
        return mode.getFocussed();
    }
    /**
     * Bildet den <code>openList Comparator</code> fr eine fokussierende Metrik.
     */
    public class FocussedComparator implements Comparator<AStarWaypoint>{
        public int compare(AStarWaypoint o1, AStarWaypoint o2){
            return o1.getProbedDistance()-o2.getProbedDistance();
        }
        public boolean equals(Object o){
            return this.equals(o);
        }
    }
    /**
     * Bildet den <code>openList Comparator</code> fr eine nicht-fokussierende Metrik.
     */
    public class UnfocussedComparator implements Comparator<PathfindingWaypoint>{
        public int compare(PathfindingWaypoint o1, PathfindingWaypoint o2){
            return o1.getTargetDistance()-o2.getTargetDistance();
        }
        public boolean equals(Object o){
            return this.equals(o);
        }
    }
    /**
     * Berechnet einen optimalen Pfad von Punkt A nach Punkt B.
     */
    public class AStarAlgorithm implements PathfindingAlgorithm{
        public void init(){
            openList = new PriorityQueue<AStarWaypoint>(11, new FocussedComparator());
            closedList = new HashSet<AStarWaypoint>();
            way = new Stack<Coordinate>();
            mode = ASTAR;
        }
        public boolean computePathTo(int x, int y){
            boolean result = false;
            if(operator.getWorld().getObjectsAt(x, y, Obstacle.class).size() == 0 && x < operator.getWorld().getWidth() && x >= 0 && y < operator.getWorld().getHeight() && y >= 0){
                AStarWaypoint starter = new AStarWaypoint(null, 0, instance);
                operator.getWorld().addObject(starter, x, y);
                openList.add(starter);
                while(!gotPath && !openList.isEmpty()){
                    AStarWaypoint wpt = (AStarWaypoint) openList.poll();
                    wpt.expand();
                    if(wpt.isStart()){
                        result = true;
                        way = wpt.getWayAsStack(way);
                        gotPath = true;
                    }
                }
                operator.getWorld().removeObjects(openList);
                operator.getWorld().removeObjects(closedList);
                openList.clear();
                closedList.clear();
            }
            return result;
        }
        public Coordinate step(){
            Coordinate next = null;
            if(!way.isEmpty()){
                next = way.pop();
            }else{
                gotPath = false;
            }
            return next;
        }
        public void setFocussed(boolean focussed){
            if(focussed){openList = new PriorityQueue<AStarWaypoint>(11, new FocussedComparator());}else{openList = new PriorityQueue<AStarWaypoint>(11, new UnfocussedComparator());}
        }
        public boolean getFocussed(){
            return openList.comparator() instanceof FocussedComparator;
        }
    }
}
