package org.barney.greenfoot.pathfinding;
import org.barney.greenfoot.extensions.Pathfinder;
import org.barney.greenfoot.world.*;
import org.barney.greenfoot.core.*;
import greenfoot.*;  // (World, Actor, GreenfootImage, Greenfoot and MouseInfo)
import java.util.*;

/**
 * Stellt einen Wegpunkt fr Pfadberechnungen dar.
 * 
 * @author  Jan-Niklas Brandes
 * @version  1.0
 */
public class AStarWaypoint extends PathfindingWaypoint{
    private Pathfinder finder;
    private int targetDist;
    private int probedDist;
    private AStarWaypoint preduccessor;
    /**
     * Instantiiert einen Wegpunkt mit A*-Funktionalitt.
     * @param  preduccessor Der Vorgnger dieses Punktes.
     * @param  targetDist Die Distanz zum Ziel.
     * @param  pathfinder Das berechnende <code>Pathfinder</code>-Modul.
     */
    public AStarWaypoint(AStarWaypoint preduccessor, int targetDist, Pathfinder pathfinder){
        this.preduccessor = preduccessor;
        this.targetDist = targetDist;
        this.finder = pathfinder;
    }
    /**
     * Berechnet die Minimaldistanz von diesem Punkt zum Start, sobald eine <code>World</code>-Referenz vorhanden ist.
     * @param  w Die Welt dieses Punktes.
     */
    public void addedToWorld(World w){
        probedDist = new Float((this.getX()-finder.getOperator().getX())*Math.signum(this.getX()-finder.getOperator().getX())+(this.getY()-finder.getOperator().getY())*Math.signum(this.getY()-finder.getOperator().getY())+new Integer(targetDist).floatValue()).intValue();
    }
    public void expand(){
        for(Object obj : this.getNeighbours(1, false, AStarWaypoint.class)){
            AStarWaypoint wpt = (AStarWaypoint) obj;
            if(wpt.getTargetDistance() > targetDist+1 && !finder.closedList.contains(wpt)){
                wpt.update(this, targetDist+1);
            }
        }
        for(int i = 0; i < 4; i++){
            int x = this.getX()+(int)Math.sin(Math.toRadians(i*90));
            int y = this.getY()+(int)Math.sin(Math.toRadians((i+1)*90));
            if(this.getWorld().getObjectsAt(x, y, Obstacle.class).size() == 0 && x < this.getWorld().getWidth() && x >= 0 && y < this.getWorld().getHeight() && y >= 0 && this.getWorld().getObjectsAt(x, y, AStarWaypoint.class).size() == 0){
                AStarWaypoint newWaypoint = new AStarWaypoint(this, targetDist, finder);
                this.getWorld().addObject(newWaypoint, x, y);
                finder.openList.add(newWaypoint);
            }
        }
        finder.closedList.add(this);
    }
    /**
     * Gibt zurck, ob dieser Punkt auf dem Startpunkt des <code>Operator</code>-Objekts liegt.
     * @return  Ob <code>Operator</code> und Punkt auf demselben Feld liegen.
     */
    public boolean isStart(){
        return this.getX() == finder.getOperator().getX() && this.getY() == finder.getOperator().getY();
    }
    /**
     * Gibt den Pfad von diesem Punkt zum Ziel als <code>Stack</code> zurck.
     * @param  stack Ein leerer Stack, in dem die Punktkoordinaten gespeichert werden.
     * @return  Den mit Koordinaten gefllten Stack.
     */
    public Stack<Coordinate> getWayAsStack(Stack<Coordinate> stack){
        if(preduccessor != null){
            preduccessor.getWayAsStack(stack);
        }
        stack.add(new Coordinate(this.getX(), this.getY()));
        return stack;
    }
    public int getTargetDistance(){
        return targetDist;
    }
    /**
     * Aktualisiert die alten Werte dieses Punktes.
     * @param  newPreduccessor Der neue Vorgnger dieses Punktes.
     * @param  newDist Die neue Distanz zum Ziel.
     */
    public void update(AStarWaypoint newPreduccessor, int newDist){
        this.preduccessor = newPreduccessor;
        this.targetDist = newDist;
        probedDist = new Float((this.getX()-finder.getOperator().getX())*Math.signum(this.getX()-finder.getOperator().getX())+(this.getY()-finder.getOperator().getY())*Math.signum(this.getY()-finder.getOperator().getY())+new Integer(targetDist).floatValue()).intValue();
    }
    /**
     * Gibt die Minimaldistanz von diesem Punkt zum Start zurck.
     */
    public int getProbedDistance(){
        return probedDist;
    }
}