package org.barney.greenfoot.pathfinding;
import org.barney.greenfoot.world.*;
/**
 * Defininiert die Standardmethoden eines Pfadfinder-Algorithmus.
 * 
 * @author Jan-Niklas Brandes 
 * @version 1.0
 */
public interface PathfindingAlgorithm  
{
    /**
     * Berechnet einen Pfad zu den Koordinaten (x|y).
     * @param  x Der x-Wert der Zielkoordinate.
     * @param  y Der y-Wert der Zielkoordinate.
     * @return  Ob ein Pfad gefunden wurde.
     */
    public boolean computePathTo(int x, int y);
    /**
     * Gibt die Koordinate des nchsten Schrittes aus.
     * @return  Die nchste Koordinate.
     */
    public Coordinate step();
    /**
     * Stellt ein, ob eine fokussierende Metrik verwendet werden soll.
     * @param  focussed Ob eine Heuristik verwendet werden soll.
     */
    public void setFocussed(boolean focussed);
    /**
     * Gibt zurck, ob eine fokussierende Metrik verwendet wird.
     * @return  Ob eine Heuristik verwendet wird.
     */
    public boolean getFocussed();
    /**
     * Bereitet den Algorithmus auf Berechnungen vor. 
     * Diese Methode muss nach der Instantiierung eines Algorithmus aufgerufen werden.
     */
    public void init();
}
